//
//  SFTPViewController.swift
//  SymbolFontToPDF
//
//  Created by narita on 2019/06/28.
//  Copyright © 2019 narita. All rights reserved.
//

import Cocoa

class SFTPViewController: NSViewController, MINTLDropZoneDelegate
{
    // MARK: - class methods
    
    // MARK: - init methods
    
    // MARK: - dealloc
    
    // MARK: - NSCopying, hash, isEqual:
    
    // MARK: - NSCoding methods
    
    // MARK: - restorableState methods
    
    // MARK: - life cycle methods
    
    // MARK: - action methods

    override func viewDidLoad()
    {
        super.viewDidLoad()

        // Do any additional setup after loading the view.
        
        self.dropZone.registerForDraggedTypes([.rtf])
        
        self.dropZone.fillColor = NSColor(patternImage: NSImage(named: "pattern.png")! )

        self.updateImage()
    }

    // MARK: - action methods
    
    @IBAction func exportPNG(_ sender: Any?)
    {
        let theSavePanel = NSSavePanel()
        
        theSavePanel.allowedFileTypes = ["png"]
        
        theSavePanel.begin
        {
            (inResult) in
            
            if inResult == .OK
            {
                let theCGRef = self.imageView.image!.cgImage(forProposedRect: nil,
                                                             context: nil,
                                                             hints: nil)
                
                let theNewRep = NSBitmapImageRep(cgImage: theCGRef!)
                theNewRep.size = self.imageView.image!.size
                
                let theData = theNewRep.representation(using: .png,
                                                       properties: [:])
                
                try! theData!.write(to: theSavePanel.url!)
            }
        }
    }

    @IBAction func exportPDF(_ sender: Any?)
    {
        let theSavePanel = NSSavePanel()
        
        theSavePanel.allowedFileTypes = ["pdf"]
        
        theSavePanel.begin
        {
            (inResult) in
            
            if inResult == .OK
            {
                let theData = self.imageView.dataWithPDF(inside: self.imageView.bounds)

                try! theData.write(to: theSavePanel.url!)
            }
        }
    }

    // MARK: - event handling methods
    
    // MARK: - drawing methods

    @objc func makeImage(string: NSString, font: NSFont, size: NSSize) -> NSImage
    {
        let theImage = NSImage(size: size)

        let theImageRect = NSMakeRect(0, 0, size.width, size.height)

        let thePointSize = self.calcPointSize(string: string, font: self.font, size: size)
        
        theImage.lockFocus()
        
        let theFont = NSFont(name: self.font.fontName,
                             size: thePointSize)

        
        let theDrawSize = string.size(withAttributes: [ NSAttributedString.Key.font: theFont!])
        
        let theStringOrigin = NSMakePoint(theImageRect.origin.x + (theImageRect.size.width - theDrawSize.width) / 2,
                                          theImageRect.origin.y + (theImageRect.size.height - theDrawSize.height) / 2)
        
        string.draw(at: theStringOrigin, withAttributes: [ NSAttributedString.Key.font: theFont!])
        
        theImage.unlockFocus()
        
        return theImage
    }
    
    // Sizeに入る最大のフォントのサイズのNSFontを算出する
    @objc func calcPointSize(string: NSString, font: NSFont, size: NSSize) -> CGFloat
    {
        let theFontName = font.fontName
        var theFontSize = font.pointSize
        
        while(true)
        {
            let theFont = NSFont(name: theFontName,
                                 size: theFontSize)

            let theSize = string.size(withAttributes: [ NSAttributedString.Key.font: theFont!])

            if theSize.width >= size.height || theSize.height >= size.height
            {
                return theFont!.pointSize - 0.1
            }
            
            theFontSize = theFontSize + 0.1
        }
    }
        
    @objc func updateImage()
    {
        let theSize = self.imageSize
        
        let theImage = self.makeImage(string: self.string,
                                      font: self.font,
                                      size: theSize)
    
        self.imageView.image = theImage
    }
    
    // MARK: - delegate/datasource methods
    
    // ドロップできるか、できるならばどのDragOperationかを返す。できない場合は[]を返す。
    @objc func dropZone(_ dropZone: MINTLDropZone, validateDrop info: NSDraggingInfo) -> NSDragOperation
    {
        if info.draggingPasteboard.types!.contains(.rtf)
        {
            return [.generic]
        }
        
        return []
    }
    
    // ドロップされたと時のカスタマイズに使う、Dropを受け入れたならばtrueを返す。
    // delegateでDropされた時の各種処理を行う。
    @objc func dropZone(_ dropZone: MINTLDropZone, acceptDrop info: NSDraggingInfo) -> Bool
    {
        if info.draggingPasteboard.types!.contains(.rtf)
        {
            if let theData = info.draggingPasteboard.data(forType: .rtf)
            {
                if let theString = NSAttributedString(rtf: theData, documentAttributes:nil)
                {
                    self.string = theString.string as NSString
                    
                    // 最初のフォント属性を取得する
                    let theAttr = theString.attributes(at: 0, effectiveRange:nil)
                    self.font = theAttr[ .font ] as! NSFont
                    
                    self.updateImage()

                    return true
                }
            }
        }
        
        return false
    }
    
    // MARK: - accessor methods (in pairs)

    @objc @IBOutlet var dropZone: MINTLDropZone! = nil
    @objc @IBOutlet var imageView: NSImageView! = nil
    
    @objc var imageSize: NSSize
    {
        switch imageSizeType
        {
        case 0:
            return NSSize(width: 512, height: 512)
        case 1:
            return NSSize(width: 256, height: 256)
        case 2:
            return NSSize(width: 128, height: 128)
        case 3:
            return NSSize(width: 64, height: 64)
        case 4:
            return NSSize(width: 32, height: 32)
        case 5:
            return NSSize(width: 16, height: 16)
        default:
            return NSSize(width: 0, height: 0)
        }
    }
    
    @objc var imageSizeType: NSInteger = 1
    {
        didSet
        {
            // フレームを変更して中央へ移動
            let theSuperView = self.imageView.superview!
            let theOrigin = NSMakePoint((theSuperView.frame.size.width - self.imageSize.width) / 2,
                                        (theSuperView.frame.size.height - self.imageSize.height) / 2)

            self.imageView.frame.size = self.imageSize
            self.imageView.frame.origin = theOrigin

            self.updateImage()
        }
    }
    
    @objc var string: NSString = "B"
    
    @objc var font:   NSFont   = { return NSFont.systemFont(ofSize: /*NSFont.systemFontSize*/60) }()
    
    override var representedObject: Any?
    {
        didSet
        {
            // Update the view, if already loaded.
        }
    }

    // MARK: - Utility methods

    
    
}

